"""
    release
    ~~~~~~~~~~~~~~~~~~~

    Make github a release using github api

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

from gease.rest import Api
from gease.uritemplate import UriTemplate
import gease.exceptions as exceptions


RELEASE_URL = 'https://api.github.com/repos{/owner}{/repo}/releases'
KEY_HTML_URL = 'html_url'
MESSAGE_MISSING_KEY = 'No %s in github repsonse' % KEY_HTML_URL


class EndPoint(object):
    """
    Github release endpoint

    More documentation is available at
    https://developer.github.com/v3/repos/releases/
    """
    def __init__(self, token, owner, repo):
        self.__template = UriTemplate(RELEASE_URL)
        self.__template.owner = owner
        self.__template.repo = repo
        self.__client = Api(token)

    @property
    def url(self):
        return str(self.__template)

    def publish(self, **kwargs):
        """
        Publish the release

        :returns: the url to the release on github
        :throws: exception if gihub changes its api response
        """
        try:
            json_reply = self.__client.create(self.url, kwargs)
            return json_reply[KEY_HTML_URL]
        except KeyError:
            raise exceptions.AbnormalGithubResponse(
                MESSAGE_MISSING_KEY)
        except exceptions.ReleaseExistException:
            raise exceptions.AbnormalGithubResponse(
                "Release or tag %s exists" % kwargs['tag_name'])
        except exceptions.RepoNotFoundError:
            raise exceptions.AbnormalGithubResponse(
                "Repository %s does not exist" % self.__template.repo)
        except exceptions.UnhandledException as e:
            raise exceptions.AbnormalGithubResponse(str(e))



class ReleaseExistException(Exception):
    pass


class RepoNotFoundError(Exception):
    pass


class GitHubException(Exception):
    pass


class NoGeaseConfigFound(Exception):
    pass


class AbnormalGithubResponse(Exception):
    pass


class UnhandledException(Exception):
    pass



"""
    gease
    ~~~~~~~~~~~~~~~~~~~

    Make github release at command line

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

import os
import sys
import json
import crayons
from gease._version import __version__, __description__
from gease.release import EndPoint
import gease.exceptions as exceptions
try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError

HELP = """%s. version %s

Usage: %s

where:

   release message is optional. It could be a quoted string or space separate
   string

Examples:

   gs gease v0.0.1 first great release
   gs gease v0.0.2 "second great release"
""" % (
    crayons.yellow('gease ' + __description__),
    crayons.magenta(__version__, bold=True),
    crayons.yellow('gs repo tag [release message]', bold=True),
)
DEFAULT_GEASE_FILE_NAME = '.gease'
DEFAULT_RELEASE_MESSAGE = "A new release via gease."
NOT_ENOUGH_ARGS = 'Not enough arguments'
KEY_GEASE_USER = 'user'
KEY_GEASE_TOKEN = 'personal_access_token'
MESSAGE_FMT_RELEASED = 'Release is created at: %s'


def main():
    if len(sys.argv) < 3:
        if len(sys.argv) == 2:
            error(NOT_ENOUGH_ARGS)
        print(HELP)
        sys.exit(-1)
    try:
        user, token = get_token()
    except exceptions.NoGeaseConfigFound as e:
        fatal(str(e))
    except KeyError as e:
        fatal("Key %s is not found" % str(e))

    repo = sys.argv[1]
    tag = sys.argv[2]
    msg = " ".join(sys.argv[3:])
    if len(msg) == 0:
        msg = DEFAULT_RELEASE_MESSAGE
    release = EndPoint(token, user, repo)
    try:
        url = release.publish(tag_name=tag, name=tag, body=msg)
        print(MESSAGE_FMT_RELEASED % crayons.green(url))
    except exceptions.AbnormalGithubResponse as e:
        error(str(e))


def get_token():
    """
    Find geasefile from user's home folder
    """
    home_dir = os.path.expanduser('~')
    geasefile = os.path.join(home_dir, DEFAULT_GEASE_FILE_NAME)
    try:
        with open(geasefile, 'r') as config:
            gease = json.load(config)
            return gease[KEY_GEASE_USER], gease[KEY_GEASE_TOKEN]
    except FileNotFoundError:
        raise exceptions.NoGeaseConfigFound(
            'Cannot find %s' % geasefile)


def error(message):
    print('Error: %s' % crayons.red(message))


def fatal(message):
    error(message)
    sys.exit(-1)


if __name__ == '__main__':
    main()



"""
    rest
    ~~~~~~~~~~~~~~~~~~~

    Only use post interface

    :copyright: (c) 2017 by Onni Software Ltd.
    :license: MIT License, see LICENSE for more details

"""

import requests
import gease.exceptions as exceptions


class Api(object):
    """
    A session holder so that each request shares the same token
    """
    def __init__(self, personal_access_token):
        self.__session = requests.Session()
        self.__session.headers.update({
            'Authorization': 'token %s' % personal_access_token})

    def create(self, url, data):
        """
        Create a release

        More information at:
        https://developer.github.com/v3/repos/releases/#create-a-release
        """
        r = self.__session.post(url, json=data)
        if r.status_code == 201:
            return r.json()
        elif r.status_code == 422:
            raise exceptions.ReleaseExistException()
        elif r.status_code == 401:
            response = r.json()
            message = '%s. Please check your gease file' % response['message']
            raise exceptions.AbnormalGithubResponse(message)
        elif r.status_code == 404:
            raise exceptions.RepoNotFoundError()
        else:
            message = 'Github responded with HTTP %s, %s ' % (
                r.status_code, r.text)
            raise exceptions.UnhandledException(message)
